"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.partitionMap = exports.partition = exports.filterMap = exports.filter = void 0;
var function_1 = require("./function");
var Option_1 = require("./Option");
var Predicate_1 = require("./Predicate");
var Separated_1 = require("./Separated");
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `filter` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var filter = function (F, G) { return function (predicate) { return function (fga) { return (0, function_1.pipe)(fga, F.map(G.filter(predicate))); }; }; };
exports.filter = filter;
/**
 * `filterMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var filterMap = function (F, G) {
    return function (f) { return F.map(G.filterMap(f)); };
};
exports.filterMap = filterMap;
/**
 * `partition` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var partition = function (F, G) {
    var _filter = (0, exports.filter)(F, G);
    return function (predicate) {
        var left = _filter((0, Predicate_1.not)(predicate));
        var right = _filter(predicate);
        return function (fga) {
            return (0, Separated_1.separated)(left(fga), right(fga));
        };
    };
};
exports.partition = partition;
/**
 * `partitionMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var partitionMap = function (F, G) {
    var _filterMap = (0, exports.filterMap)(F, G);
    return function (f) { return function (fga) { return (0, Separated_1.separated)((0, function_1.pipe)(fga, _filterMap((0, function_1.flow)(f, Option_1.getLeft))), (0, function_1.pipe)(fga, _filterMap((0, function_1.flow)(f, Option_1.getRight)))); }; };
};
exports.partitionMap = partitionMap;
