"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.let = exports.tupled = exports.bindTo = exports.map = exports.flap = void 0;
/**
 * A `Functor` is a type constructor which supports a mapping operation `map`.
 *
 * `map` can be used to turn functions `A -> B` into functions `F<A> -> F<B>` whose argument and return types use the type
 * constructor `F` to represent some computational context.
 *
 * Instances must satisfy the following laws:
 *
 * 1. Identity: `map(identity) <-> identity`
 * 2. Composition: `map(flow(ab, bc)) <-> flow(map(ab), map(bc))`
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
var tuple_1 = require("./tuple");
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
var flap = function (F) { return function (a) { return F.map((0, function_1.apply)(a)); }; };
exports.flap = flap;
/**
 * `map` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var map = function (F, G) { return function (f) { return F.map(G.map(f)); }; };
exports.map = map;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
var bindTo = function (F) { return function (name) {
    return F.map(function (a) {
        var _a;
        return (_a = {}, _a[name] = a, _a);
    });
}; };
exports.bindTo = bindTo;
/**
 * @since 3.0.0
 */
var tupled = function (F) { return F.map(tuple_1.tuple); };
exports.tupled = tupled;
var let_ = function (F) {
    return function (name, f) { return F.map(function (a) {
        var _a;
        return Object.assign({}, a, (_a = {}, _a[name] = f(a), _a));
    }); };
};
exports.let = let_;
