"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.right = exports.left = exports.flap = exports.Functor = exports.map = exports.Bifunctor = exports.mapLeft = exports.bimap = exports.separated = void 0;
/**
 * @since 3.0.0
 */
var Bifunctor_1 = require("./Bifunctor");
var Functor_1 = require("./Functor");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
var separated = function (left, right) { return ({ left: left, right: right }); };
exports.separated = separated;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
var bimap = function (f, g) { return function (fa) { return (0, exports.separated)(f((0, exports.left)(fa)), g((0, exports.right)(fa))); }; };
exports.bimap = bimap;
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
exports.mapLeft = (0, Bifunctor_1.mapLeftDefault)(exports.bimap);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
exports.Bifunctor = {
    bimap: exports.bimap,
    mapLeft: exports.mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
exports.map = (0, Bifunctor_1.mapDefault)(exports.bimap);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = (0, Functor_1.flap)(exports.Functor);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
var left = function (s) { return s.left; };
exports.left = left;
/**
 * @since 3.0.0
 */
var right = function (s) { return s.right; };
exports.right = right;
