import { constVoid, flow, pipe } from './function';
import { map } from './Functor';
import { getLeft, getRight } from './Option';
import { right, separated } from './Separated';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// defaults
// -------------------------------------------------------------------------------------
/**
 * Return a default `compact` implementation from `Functor` and `separate`.
 *
 * @category defaults
 * @since 3.0.0
 */
export const compactDefault = (F) => (separate) => {
    return flow(F.map(_.fromOption(constVoid)), separate, right);
};
/**
 * Return a default `separate` implementation from `Functor` and `compact`.
 *
 * @category defaults
 * @since 3.0.0
 */
export function separateDefault(F) {
    return (compact) => (fe) => separated(pipe(fe, F.map(getLeft), compact), pipe(fe, F.map(getRight), compact));
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `compact` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export function compact(F, G) {
    return F.map(G.compact);
}
/**
 * `separate` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export function separate(F, C, G) {
    const compactFC = compact(F, C);
    const mapFG = map(F, G);
    return (fge) => separated(pipe(fge, mapFG(getLeft), compactFC), pipe(fge, mapFG(getRight), compactFC));
}
