/**
 * @since 3.0.0
 */
import { ap as ap_ } from './Apply';
import * as E from './Either';
import { flow, pipe } from './function';
import { map as map_ } from './Functor';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const right = (F) => (a) => F.of(E.right(a));
/**
 * @since 3.0.0
 */
export const left = (F) => (e) => F.of(E.left(e));
/**
 * @since 3.0.0
 */
export function rightF(F) {
    return F.map(E.right);
}
/**
 * @since 3.0.0
 */
export function leftF(F) {
    return F.map(E.left);
}
/**
 * @since 3.0.0
 */
export const fromNullable = (F) => (e) => (a) => F.of(pipe(a, E.fromNullable(() => e)));
/**
 * @since 3.0.0
 */
export const fromNullableK = (F) => {
    const fromNullableF = fromNullable(F);
    return (e) => {
        const fromNullableFE = fromNullableF(e);
        return (f) => (...a) => fromNullableFE(f(...a));
    };
};
/**
 * @since 3.0.0
 */
export const chainNullableK = (M) => {
    const chainM = chain(M);
    const fromNullableKM = fromNullableK(M);
    return (e) => {
        const fromNullableKMe = fromNullableKM(e);
        return (f) => (ma) => pipe(ma, chainM(fromNullableKMe(f)));
    };
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const map = (F) => map_(F, E.Functor);
/**
 * @since 3.0.0
 */
export const ap = (F) => {
    return ap_(F, E.Apply);
};
/**
 * @since 3.0.0
 */
export const chain = (M) => (f) => (ma) => {
    return pipe(ma, M.chain((e) => (E.isLeft(e) ? M.of(e) : f(e.right))));
};
/**
 * @since 3.0.0
 */
export const alt = (M) => (second) => (first) => {
    return pipe(first, M.chain((e) => E.isLeft(e) ? second() : M.of(e)));
};
/**
 * @since 3.0.0
 */
export function bimap(F) {
    return flow(E.bimap, F.map);
}
/**
 * @since 3.0.0
 */
export function mapLeft(F) {
    return (f) => F.map(E.mapLeft(f));
}
/**
 * @since 3.0.0
 */
export const altValidation = (M, S) => (second) => (first) => {
    const rightM = right(M);
    return pipe(first, M.chain(E.match((e1) => pipe(second(), M.map(E.mapLeft((e2) => S.concat(e2)(e1)))), rightM)));
};
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function match(F) {
    return flow(E.match, F.map);
}
/**
 * @since 3.0.0
 */
export const matchE = (M) => (onLeft, onRight) => {
    return M.chain(E.match(onLeft, onRight));
};
/**
 * @since 3.0.0
 */
export const getOrElse = (F) => (onLeft) => {
    return F.map(E.getOrElse(onLeft));
};
/**
 * @since 3.0.0
 */
export const getOrElseE = (M) => (onLeft) => (ma) => {
    return pipe(ma, M.chain(E.match(onLeft, M.of)));
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const orElse = (M) => (onLeft) => (ma) => {
    return pipe(ma, M.chain((e) => E.isLeft(e) ? onLeft(e.left) : M.of(e)));
};
/**
 * @since 3.0.0
 */
export const orLeft = (M) => (onLeft) => (fa) => {
    return pipe(fa, M.chain(E.match((e) => pipe(onLeft(e), M.map(E.left)), (a) => M.of(E.right(a)))));
};
/**
 * @since 3.0.0
 */
export const orElseFirst = (M) => {
    const orElseM = orElse(M);
    return (onLeft) => (ma) => {
        return pipe(ma, orElseM((e) => pipe(onLeft(e), M.map((eb) => (E.isLeft(eb) ? eb : E.left(e))))));
    };
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function swap(F) {
    return F.map(E.swap);
}
/**
 * @since 3.0.0
 */
export function toUnion(F) {
    return F.map(E.toUnion);
}
/**
 * @since 3.0.0
 */
export const bracket = (M) => (acquire, use, release) => {
    const leftM = left(M);
    return pipe(acquire, M.chain(E.match(leftM, (a) => pipe(use(a), M.chain((e) => pipe(release(a, e), M.chain(E.match(leftM, () => M.of(e)))))))));
};
