import { flow, pipe } from './function';
import { getLeft, getRight } from './Option';
import { not } from './Predicate';
import { separated } from './Separated';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `filter` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const filter = (F, G) => (predicate) => (fga) => pipe(fga, F.map(G.filter(predicate)));
/**
 * `filterMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = (F, G) => {
    return (f) => F.map(G.filterMap(f));
};
/**
 * `partition` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const partition = (F, G) => {
    const _filter = filter(F, G);
    return (predicate) => {
        const left = _filter(not(predicate));
        const right = _filter(predicate);
        return (fga) => separated(left(fga), right(fga));
    };
};
/**
 * `partitionMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = (F, G) => {
    const _filterMap = filterMap(F, G);
    return (f) => (fga) => separated(pipe(fga, _filterMap(flow(f, getLeft))), pipe(fga, _filterMap(flow(f, getRight))));
};
