/**
 * The `FromEither` type class represents those data types which support errors.
 *
 * @since 3.0.0
 */
import { chainFirst } from './Chain';
import { pipe } from './function';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromOption = (F) => (onNone) => {
    const fromOption = _.fromOption(onNone);
    return (ma) => F.fromEither(fromOption(ma));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = (F) => (predicate) => (a) => F.fromEither(predicate(a) ? _.right(a) : _.left(a));
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = (F) => {
    const fromOptionF = fromOption(F);
    return (onNone) => {
        const from = fromOptionF(onNone);
        return (f) => (...a) => from(f(...a));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionK = (F, M) => {
    const fromOptionKF = fromOptionK(F);
    return (onNone) => {
        const from = fromOptionKF(onNone);
        return (f) => (ma) => pipe(ma, M.chain(from(f)));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = (F) => (f) => (...a) => F.fromEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = (F, M) => {
    const fromEitherKF = fromEitherK(F);
    return (f) => (ma) => {
        return pipe(ma, M.chain(fromEitherKF(f)));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = (F, M) => {
    const chainFirstM = chainFirst(M);
    const fromEitherKF = fromEitherK(F);
    return (f) => {
        const fromEitherKFf = fromEitherKF(f);
        return chainFirstM((a) => fromEitherKFf(a));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export function filterOrElse(F, M) {
    return (predicate, onFalse) => (ma) => pipe(ma, M.chain((a) => F.fromEither(predicate(a) ? _.right(a) : _.left(onFalse(a)))));
}
