/**
 * @since 3.0.0
 */
import { ap as ap_ } from './Apply';
import { constant, flow, pipe } from './function';
import { map as map_ } from './Functor';
import * as _ from './internal';
import * as O from './Option';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const some = (F) => (a) => F.of(_.some(a));
/**
 * @since 3.0.0
 */
export function zero(F) {
    return constant(F.of(_.none));
}
/**
 * @since 3.0.0
 */
export function fromF(F) {
    return F.map(_.some);
}
/**
 * @since 3.0.0
 */
export const fromNullable = (F) => (a) => F.of(O.fromNullable(a));
/**
 * @since 3.0.0
 */
export function fromNullableK(F) {
    const fromNullableF = fromNullable(F);
    return (f) => (...a) => fromNullableF(f(...a));
}
/**
 * @since 3.0.0
 */
export const fromOptionK = (F) => (f) => (...a) => F.of(f(...a));
/**
 * @since 3.0.0
 */
export function fromPredicate(F) {
    return (predicate) => {
        const fromPredicate = O.fromPredicate(predicate);
        return (a) => F.of(fromPredicate(a));
    };
}
/**
 * @since 3.0.0
 */
export const fromEither = (F) => (e) => F.of(O.fromEither(e));
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function match(F) {
    return flow(O.match, F.map);
}
/**
 * @since 3.0.0
 */
export const matchE = (M) => (onNone, onSome) => {
    return M.chain(O.match(onNone, onSome));
};
/**
 * @since 3.0.0
 */
export const getOrElse = (F) => (onNone) => {
    return F.map(O.getOrElse(onNone));
};
/**
 * @since 3.0.0
 */
export const getOrElseE = (M) => (onNone) => (ma) => {
    return pipe(ma, M.chain(O.match(onNone, M.of)));
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const chainNullableK = (M) => {
    const chainM = chain(M);
    const fromNullableKM = fromNullableK(M);
    return (f) => {
        const fromNullableKMf = fromNullableKM(f);
        return chainM((a) => fromNullableKMf(a));
    };
};
/**
 * @since 3.0.0
 */
export const chainOptionK = (M) => {
    const chainM = chain(M);
    const fromOptionKM = fromOptionK(M);
    return (f) => {
        const fromOptionKf = fromOptionKM(f);
        return chainM((a) => fromOptionKf(a));
    };
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function map(F) {
    return map_(F, O.Functor);
}
/**
 * @since 3.0.0
 */
export const ap = (F) => {
    return ap_(F, O.Apply);
};
/**
 * @since 3.0.0
 */
export const chain = (M) => (f) => (ma) => {
    return pipe(ma, M.chain(O.match(() => zero(M)(), f)));
};
/**
 * @since 3.0.0
 */
export const alt = (M) => (second) => (first) => {
    return pipe(first, M.chain(O.match(second, some(M))));
};
