import { fromWriterK as fromWriterK_ } from './FromWriter';
import { flow } from './function';
import { bindTo as bindTo_, flap as flap_, let as let__, tupled as tupled_ } from './Functor';
import * as R from './Reader';
import * as RT from './ReaderTask';
import * as T from './Task';
import * as WT from './WriterT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReader = (w) => (fa) => fromReaderTask(w)(RT.fromReader(fa));
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderTask = /*#__PURE__*/ WT.fromF(RT.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTaskWriter = 
/*#__PURE__*/ R.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIO = /*#__PURE__*/ WT.fromIO(RT.Functor, RT.FromIO);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTask = /*#__PURE__*/ WT.fromTask(RT.Functor, RT.FromTask);
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export const tell = /*#__PURE__*/ WT.tell(RT.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderTaskWriter = R.asksReader;
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromWriter = (w) => RT.of(w);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReaderWriter = (fa) => flow(fa, T.of);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const fst = /*#__PURE__*/ WT.fst(RT.Functor);
/**
 * @since 3.0.0
 */
export const snd = /*#__PURE__*/ WT.snd(RT.Functor);
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export const evaluate = fst;
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export const execute = snd;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = R.local;
/**
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ WT.swap(RT.Functor);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskWriterK = (f) => (...a) => fromTaskWriter(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderWriterK = (f) => flow(f, fromReaderWriter);
/**
 * @since 3.0.0
 */
export const listen = /*#__PURE__*/ WT.listen(RT.Functor);
/**
 * @since 3.0.0
 */
export const pass = /*#__PURE__*/ WT.pass(RT.Functor);
/**
 * @since 3.0.0
 */
export const listens = /*#__PURE__*/ WT.listens(RT.Functor);
/**
 * @since 3.0.0
 */
export const censor = /*#__PURE__*/ WT.censor(RT.Functor);
// -------------------------------------------------------------------------------------
// type class operations
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ WT.map(RT.Functor);
/**
 * @category type class operations
 * @since 3.0.0
 */
export const mapLeft = /*#__PURE__*/ WT.mapLeft(RT.Functor);
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const bimap = /*#__PURE__*/ WT.bimap(RT.Functor);
/**
 * Maps a function over the first component of a `Writer`.
 *
 * Alias of [`map`](#map)
 *
 * @since 3.0.0
 */
export const mapFst = map;
/**
 * Maps a function over the second component of a `Writer`.
 *
 * Alias of [`mapLeft`](#mapleft)
 *
 * @since 3.0.0
 */
export const mapSnd = mapLeft;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft: mapSnd
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: WT.of(RT.Pointed, M)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (A, S) => ({
    map,
    ap: WT.ap(A, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (A, M) => {
    const { ap } = getApply(A, M);
    const P = getPointed(M);
    return {
        map,
        ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => {
    return {
        map,
        chain: WT.chain(RT.Chain, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getChain(M);
    return {
        map,
        of: P.of,
        chain: C.chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromWriter = {
    fromWriter
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromWriterK = /*#__PURE__*/ fromWriterK_(FromWriter);
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromReader = (M) => ({
    fromReader: fromReader(M.empty)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromIO = (M) => ({
    fromIO: fromIO(M.empty)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromTask = (M) => ({
    fromIO: fromIO(M.empty),
    fromTask: fromTask(M.empty)
});
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const bindTo = /*#__PURE__*/ bindTo_(Functor);
const let_ = /*#__PURE__*/ let__(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ tupled_(Functor);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// TODO: array utils
// /**
//  * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativePar)`.
//  *
//  * @since 3.0.0
//  */
// export const traverseReadonlyNonEmptyArrayWithIndex = <A, R, E, B>(
//   f: (index: number, a: A) => ReaderTaskWriter<R, E, B>
// ): ((as: ReadonlyNonEmptyArray<A>) => ReaderTaskWriter<R, E, ReadonlyNonEmptyArray<B>>) =>
//   flow(R.traverseReadonlyNonEmptyArrayWithIndex(f), R.map(TE.traverseReadonlyNonEmptyArrayWithIndex(SK)))
// /**
//  * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
//  *
//  * @since 3.0.0
//  */
// export const traverseReadonlyArrayWithIndex = <A, R, E, B>(
//   f: (index: number, a: A) => ReaderTaskWriter<R, E, B>
// ): ((as: ReadonlyArray<A>) => ReaderTaskWriter<R, E, ReadonlyArray<B>>) => {
//   const g = traverseReadonlyNonEmptyArrayWithIndex(f)
//   return (as) => (_.isNonEmpty(as) ? g(as) : ApT)
// }
// /**
//  * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativeSeq)`.
//  *
//  * @since 3.0.0
//  */
// export const traverseReadonlyNonEmptyArrayWithIndexSeq = <A, R, E, B>(
//   f: (index: number, a: A) => ReaderTaskWriter<R, E, B>
// ): ((as: ReadonlyNonEmptyArray<A>) => ReaderTaskWriter<R, E, ReadonlyNonEmptyArray<B>>) =>
//   flow(R.traverseReadonlyNonEmptyArrayWithIndex(f), R.map(TE.traverseReadonlyNonEmptyArrayWithIndexSeq(SK)))
// /**
//  * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
//  *
//  * @since 3.0.0
//  */
// export const traverseReadonlyArrayWithIndexSeq = <A, R, E, B>(
//   f: (index: number, a: A) => ReaderTaskWriter<R, E, B>
// ): ((as: ReadonlyArray<A>) => ReaderTaskWriter<R, E, ReadonlyArray<B>>) => {
//   const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f)
//   return (as) => (_.isNonEmpty(as) ? g(as) : ApT)
// }
