import type { Functor } from './Functor'
import type { HKT, Kind } from './HKT'
import { Semigroup } from './Semigroup'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Apply<F extends HKT> extends Functor<F> {
  readonly ap: <S, R2, W2, E2, A>(
    fa: Kind<F, S, R2, W2, E2, A>
  ) => <R1, W1, E1, B>(fab: Kind<F, S, R1, W1, E1, (a: A) => B>) => Kind<F, S, R1 & R2, W1 | W2, E1 | E2, B>
}
/**
 * `ap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const ap: <F extends HKT, G extends HKT>(
  F: Apply<F>,
  G: Apply<G>
) => <FS, FR2, FW2, FE2, GS, GR2, GW2, GE2, A>(
  fa: Kind<F, FS, FR2, FW2, FE2, Kind<G, GS, GR2, GW2, GE2, A>>
) => <FR1, FW1, FE1, GR1, GW1, GE1, B>(
  fab: Kind<F, FS, FR1, FW1, FE1, Kind<G, GS, GR1, GW1, GE1, (a: A) => B>>
) => Kind<F, FS, FR1 & FR2, FW2 | FW1, FE2 | FE1, Kind<G, GS, GR1 & GR2, GW2 | GW1, GE2 | GE1, B>>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const apFirst: <F extends HKT>(
  A: Apply<F>
) => <S, R2, W2, E2, B>(
  second: Kind<F, S, R2, W2, E2, B>
) => <R1, W1, E1, A>(first: Kind<F, S, R1, W1, E1, A>) => Kind<F, S, R1 & R2, W2 | W1, E2 | E1, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const apSecond: <F extends HKT>(
  A: Apply<F>
) => <S, R2, W2, E2, B>(
  second: Kind<F, S, R2, W2, E2, B>
) => <R1, W1, E1, A>(first: Kind<F, S, R1, W1, E1, A>) => Kind<F, S, R1 & R2, W2 | W1, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const apS: <F extends HKT>(
  F: Apply<F>
) => <N extends string, A, S, R2, W2, E2, B>(
  name: Exclude<N, keyof A>,
  fb: Kind<F, S, R2, W2, E2, B>
) => <R1, W1, E1>(
  fa: Kind<F, S, R1, W1, E1, A>
) => Kind<F, S, R1 & R2, W2 | W1, E2 | E1, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const apT: <F extends HKT>(
  F: Apply<F>
) => <S, R2, W2, E2, B>(
  fb: Kind<F, S, R2, W2, E2, B>
) => <R1, W1, E1, A extends readonly unknown[]>(
  fas: Kind<F, S, R1, W1, E1, A>
) => Kind<F, S, R1 & R2, W2 | W1, E2 | E1, readonly [...A, B]>
/**
 * Lift a semigroup into 'F', the inner values are concatenated using the provided `Semigroup`.
 *
 * @since 3.0.0
 */
export declare const getApplySemigroup: <F extends HKT>(
  F: Apply<F>
) => <A, S, R, W, E>(S: Semigroup<A>) => Semigroup<Kind<F, S, R, W, E, A>>
