/**
 * @since 3.0.0
 */
import { Apply } from './Apply'
import type { Chain } from './Chain'
import * as E from './Either'
import { Lazy } from './function'
import { Functor } from './Functor'
import type { HKT, Kind } from './HKT'
import type { Monad } from './Monad'
import type { Pointed } from './Pointed'
import type { Semigroup } from './Semigroup'
import Either = E.Either
/**
 * @since 3.0.0
 */
export declare const right: <F extends HKT>(
  F: Pointed<F>
) => <A, S, R = unknown, W = never, FE = never, E = never>(a: A) => Kind<F, S, R, W, FE, E.Either<E, A>>
/**
 * @since 3.0.0
 */
export declare const left: <F extends HKT>(
  F: Pointed<F>
) => <E, S, R = unknown, W = never, FE = never, A = never>(e: E) => Kind<F, S, R, W, FE, E.Either<E, A>>
/**
 * @since 3.0.0
 */
export declare function rightF<F extends HKT>(
  F: Functor<F>
): <S, R, W, FE, A, E = never>(fa: Kind<F, S, R, W, FE, A>) => Kind<F, S, R, W, FE, Either<E, A>>
/**
 * @since 3.0.0
 */
export declare function leftF<F extends HKT>(
  F: Functor<F>
): <S, R, W, FE, E, A = never>(fe: Kind<F, S, R, W, FE, E>) => Kind<F, S, R, W, FE, Either<E, A>>
/**
 * @since 3.0.0
 */
export declare const fromNullable: <F extends HKT>(
  F: Pointed<F>
) => <E>(e: E) => <A, S, R, W, FE>(a: A) => Kind<F, S, R, W, FE, E.Either<E, NonNullable<A>>>
/**
 * @since 3.0.0
 */
export declare const fromNullableK: <F extends HKT>(
  F: Pointed<F>
) => <E>(
  e: E
) => <A extends readonly unknown[], B>(
  f: (...a: A) => B | null | undefined
) => <S, R, W, FE>(...a: A) => Kind<F, S, R, W, FE, E.Either<E, NonNullable<B>>>
/**
 * @since 3.0.0
 */
export declare const chainNullableK: <M extends HKT>(
  M: Monad<M>
) => <E>(
  e: E
) => <A, B>(
  f: (a: A) => B | null | undefined
) => <S, R, W, FE>(ma: Kind<M, S, R, W, FE, E.Either<E, A>>) => Kind<M, S, R, W, FE, E.Either<E, NonNullable<B>>>
/**
 * @since 3.0.0
 */
export declare const map: <F extends HKT>(
  F: Functor<F>
) => <A, B>(
  f: (a: A) => B
) => <S, R, W, FE, E>(fa: Kind<F, S, R, W, FE, E.Either<E, A>>) => Kind<F, S, R, W, FE, E.Either<E, B>>
/**
 * @since 3.0.0
 */
export declare const ap: <F extends HKT>(
  F: Apply<F>
) => <S, R2, W2, FE2, E2, A>(
  fa: Kind<F, S, R2, W2, FE2, E.Either<E2, A>>
) => <R1, W1, FE1, E1, B>(
  fab: Kind<F, S, R1, W1, FE1, E.Either<E1, (a: A) => B>>
) => Kind<F, S, R1 & R2, W2 | W1, FE2 | FE1, E.Either<E2 | E1, B>>
/**
 * @since 3.0.0
 */
export declare const chain: <M extends HKT>(
  M: Monad<M>
) => <A, S, R2, W2, ME2, E2, B>(
  f: (a: A) => Kind<M, S, R2, W2, ME2, E.Either<E2, B>>
) => <R1, W1, ME1, E1>(
  ma: Kind<M, S, R1, W1, ME1, E.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, E.Either<E2 | E1, B>>
/**
 * @since 3.0.0
 */
export declare const alt: <M extends HKT>(
  M: Monad<M>
) => <S, R2, W2, ME2, E2, B>(
  second: Lazy<Kind<M, S, R2, W2, ME2, E.Either<E2, B>>>
) => <R1, W1, ME1, E1, A>(
  first: Kind<M, S, R1, W1, ME1, E.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, E.Either<E2, B | A>>
/**
 * @since 3.0.0
 */
export declare function bimap<F extends HKT>(
  F: Functor<F>
): <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <S, R, W, FE>(fea: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, Either<G, B>>
/**
 * @since 3.0.0
 */
export declare function mapLeft<F extends HKT>(
  F: Functor<F>
): <E, G>(
  f: (e: E) => G
) => <S, R, W, FE, A>(fea: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, Either<G, A>>
/**
 * @since 3.0.0
 */
export declare const altValidation: <M extends HKT, E>(
  M: Monad<M>,
  S: Semigroup<E>
) => <S, R2, W2, ME2, B>(
  second: Lazy<Kind<M, S, R2, W2, ME2, E.Either<E, B>>>
) => <R1, W1, ME1, A>(
  first: Kind<M, S, R1, W1, ME1, E.Either<E, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, E.Either<E, B | A>>
/**
 * @since 3.0.0
 */
export declare function match<F extends HKT>(
  F: Functor<F>
): <E, B, A, C = B>(
  onLeft: (e: E) => B,
  onRight: (a: A) => C
) => <S, R, W, ME>(ma: Kind<F, S, R, W, ME, Either<E, A>>) => Kind<F, S, R, W, ME, B | C>
/**
 * @since 3.0.0
 */
export declare const matchE: <M extends HKT>(
  M: Chain<M>
) => <E, S, R2, W2, ME2, B, A, R3, W3, ME3, C = B>(
  onLeft: (e: E) => Kind<M, S, R2, W2, ME2, B>,
  onRight: (a: A) => Kind<M, S, R3, W3, ME3, C>
) => <R1, W1, ME1>(
  ma: Kind<M, S, R1, W1, ME1, E.Either<E, A>>
) => Kind<M, S, R1 & R2 & R3, W2 | W3 | W1, ME2 | ME3 | ME1, B | C>
/**
 * @since 3.0.0
 */
export declare const getOrElse: <F extends HKT>(
  F: Functor<F>
) => <E, B>(
  onLeft: (e: E) => B
) => <S, R, W, ME, A>(ma: Kind<F, S, R, W, ME, E.Either<E, A>>) => Kind<F, S, R, W, ME, B | A>
/**
 * @since 3.0.0
 */
export declare const getOrElseE: <M extends HKT>(
  M: Monad<M>
) => <E, S, R2, W2, ME2, B>(
  onLeft: (e: E) => Kind<M, S, R2, W2, ME2, B>
) => <R1, W1, ME1, A>(ma: Kind<M, S, R1, W1, ME1, E.Either<E, A>>) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, B | A>
/**
 * @since 3.0.0
 */
export declare const orElse: <M extends HKT>(
  M: Monad<M>
) => <E1, S, R2, W2, ME2, E2, B>(
  onLeft: (e: E1) => Kind<M, S, R2, W2, ME2, E.Either<E2, B>>
) => <R1, W1, ME1, A>(
  ma: Kind<M, S, R1, W1, ME1, E.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, E.Either<E2, B | A>>
/**
 * @since 3.0.0
 */
export declare const orLeft: <M extends HKT>(
  M: Monad<M>
) => <E1, S, R, W, ME, E2>(
  onLeft: (e: E1) => Kind<M, S, R, W, ME, E2>
) => <A>(fa: Kind<M, S, R, W, ME, E.Either<E1, A>>) => Kind<M, S, R, W, ME, E.Either<E2, A>>
/**
 * @since 3.0.0
 */
export declare const orElseFirst: <M extends HKT>(
  M: Monad<M>
) => <E1, S, R2, W2, ME2, E2, B>(
  onLeft: (e: E1) => Kind<M, S, R2, W2, ME2, E.Either<E2, B>>
) => <R1, W1, ME1, A>(
  ma: Kind<M, S, R1, W1, ME1, E.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, E.Either<E1 | E2, A>>
/**
 * @since 3.0.0
 */
export declare function swap<F extends HKT>(
  F: Functor<F>
): <S, R, W, FE, E, A>(ma: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, Either<A, E>>
/**
 * @since 3.0.0
 */
export declare function toUnion<F extends HKT>(
  F: Functor<F>
): <S, R, W, FE, E, A>(fa: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, E | A>
/**
 * @since 3.0.0
 */
export declare const bracket: <M extends HKT>(
  M: Monad<M>
) => <S, R1, W1, ME1, E1, A, R2, W2, ME2, E2, B, R3, W3, ME3, E3>(
  acquire: Kind<M, S, R1, W1, ME1, E.Either<E1, A>>,
  use: (a: A) => Kind<M, S, R2, W2, ME2, E.Either<E2, B>>,
  release: (a: A, e: E.Either<E2, B>) => Kind<M, S, R3, W3, ME3, E.Either<E3, void>>
) => Kind<M, S, R1 & R2 & R3, W1 | W2 | W3, ME1 | ME2 | ME3, E.Either<E1 | E2 | E3, B>>
