/**
 * @since 3.0.0
 */
import { HKT } from './HKT'
import type { Monoid } from './Monoid'
import type { Semigroup } from './Semigroup'
/**
 * @since 3.0.0
 */
export interface Endomorphism<A> {
  (a: A): A
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface EndomorphismF extends HKT {
  readonly type: Endomorphism<this['Invariant1']>
}
/**
 * Endomorphism form a `Semigroup` where the `concat` operation is the usual function composition.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getSemigroup: <A = never>() => Semigroup<Endomorphism<A>>
/**
 * Endomorphism form a `Monoid` where the `empty` value is the `identity` function.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <A = never>() => Monoid<Endomorphism<A>>
