/**
 * The `FromEither` type class represents those data types which support errors.
 *
 * @since 3.0.0
 */
import { Chain } from './Chain'
import type { Either } from './Either'
import { Lazy } from './function'
import type { HKT, Kind, Typeclass } from './HKT'
import type { Option } from './Option'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromEither<F extends HKT> extends Typeclass<F> {
  readonly fromEither: <E, A, S, R = unknown, W = never>(fa: Either<E, A>) => Kind<F, S, R, W, E, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromOption: <F extends HKT>(
  F: FromEither<F>
) => <E>(onNone: Lazy<E>) => <A, S, R = unknown, W = never>(fa: Option<A>) => Kind<F, S, R, W, E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <F extends HKT>(
  F: FromEither<F>
) => {
  <A, B extends A>(refinement: Refinement<A, B>): <S, R = unknown, W = never>(a: A) => Kind<F, S, R, W, A, B>
  <A_1>(predicate: Predicate<A_1>): <B_1 extends A_1, S_1, R_1 = unknown, W_1 = never>(
    b: B_1
  ) => Kind<F, S_1, R_1, W_1, B_1, B_1>
  <A_2>(predicate: Predicate<A_2>): <S_2, R_2 = unknown, W_2 = never>(a: A_2) => Kind<F, S_2, R_2, W_2, A_2, A_2>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <F extends HKT>(
  F: FromEither<F>
) => <E>(
  onNone: Lazy<E>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => Option<B>
) => <S, R = unknown, W = never>(...a: A) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainOptionK: <M extends HKT>(
  F: FromEither<M>,
  M: Chain<M>
) => <E>(
  onNone: Lazy<E>
) => <A, B>(f: (a: A) => Option<B>) => <S, R, W>(ma: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <F extends HKT>(
  F: FromEither<F>
) => <A extends readonly unknown[], E, B>(
  f: (...a: A) => Either<E, B>
) => <S, R = unknown, W = never>(...a: A) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainEitherK: <M extends HKT>(
  F: FromEither<M>,
  M: Chain<M>
) => <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <S, R, W, E1>(ma: Kind<M, S, R, W, E1, A>) => Kind<M, S, R, W, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstEitherK: <M extends HKT>(
  F: FromEither<M>,
  M: Chain<M>
) => <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <S, R, W, E1>(ma: Kind<M, S, R, W, E1, A>) => Kind<M, S, R, W, E2 | E1, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare function filterOrElse<M extends HKT>(
  F: FromEither<M>,
  M: Chain<M>
): {
  <A, B extends A, E2>(refinement: Refinement<A, B>, onFalse: (a: A) => E2): <S, R, W, E1>(
    ma: Kind<M, S, R, W, E1, A>
  ) => Kind<M, S, R, W, E1 | E2, B>
  <A, E2>(predicate: Predicate<A>, onFalse: (a: A) => E2): <S, R, W, E1, B extends A>(
    mb: Kind<M, S, R, W, E1, B>
  ) => Kind<M, S, R, W, E1 | E2, B>
  <A, E2>(predicate: Predicate<A>, onFalse: (a: A) => E2): <S, R, W, E1>(
    ma: Kind<M, S, R, W, E1, A>
  ) => Kind<M, S, R, W, E1 | E2, A>
}
