/**
 * @since 3.0.0
 */
import { Apply } from './Apply'
import type { Chain } from './Chain'
import type { Either } from './Either'
import { Lazy } from './function'
import { Functor } from './Functor'
import type { Kind, HKT } from './HKT'
import type { Monad } from './Monad'
import * as O from './Option'
import type { Pointed } from './Pointed'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import Option = O.Option
/**
 * @since 3.0.0
 */
export declare const some: <F extends HKT>(F: Pointed<F>) => <A, S, R, W, E>(a: A) => Kind<F, S, R, W, E, O.Option<A>>
/**
 * @since 3.0.0
 */
export declare function zero<F extends HKT>(F: Pointed<F>): <S, R, W, E, A>() => Kind<F, S, R, W, E, Option<A>>
/**
 * @since 3.0.0
 */
export declare function fromF<F extends HKT>(
  F: Functor<F>
): <S, R, W, E, A>(ma: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, Option<A>>
/**
 * @since 3.0.0
 */
export declare const fromNullable: <F extends HKT>(
  F: Pointed<F>
) => <A, S, R, W, E>(a: A) => Kind<F, S, R, W, E, O.Option<NonNullable<A>>>
/**
 * @since 3.0.0
 */
export declare function fromNullableK<F extends HKT>(
  F: Pointed<F>
): <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => <S, R, W, E>(...a: A) => Kind<F, S, R, W, E, Option<NonNullable<B>>>
/**
 * @since 3.0.0
 */
export declare const fromOptionK: <F extends HKT>(
  F: Pointed<F>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => O.Option<B>
) => <S, R, W, E>(...a: A) => Kind<F, S, R, W, E, O.Option<B>>
/**
 * @since 3.0.0
 */
export declare function fromPredicate<F extends HKT>(
  F: Pointed<F>
): {
  <A, B extends A>(refinement: Refinement<A, B>): <S, R, W, E>(a: A) => Kind<F, S, R, W, E, Option<B>>
  <A>(predicate: Predicate<A>): <B extends A, S, R, W, E>(b: B) => Kind<F, S, R, W, E, Option<B>>
  <A>(predicate: Predicate<A>): <S, R, W, E>(a: A) => Kind<F, S, R, W, E, Option<A>>
}
/**
 * @since 3.0.0
 */
export declare const fromEither: <F extends HKT>(
  F: Pointed<F>
) => <A, S, R, W, E>(e: Either<unknown, A>) => Kind<F, S, R, W, E, O.Option<A>>
/**
 * @since 3.0.0
 */
export declare function match<F extends HKT>(
  F: Functor<F>
): <B, A, C = B>(
  onNone: () => B,
  onSome: (a: A) => C
) => <S, R, W, E>(ma: Kind<F, S, R, W, E, Option<A>>) => Kind<F, S, R, W, E, B | C>
/**
 * @since 3.0.0
 */
export declare const matchE: <M extends HKT>(
  M: Chain<M>
) => <S, R2, W2, E2, B, A, R3, W3, E3, C = B>(
  onNone: () => Kind<M, S, R2, W2, E2, B>,
  onSome: (a: A) => Kind<M, S, R3, W3, E3, C>
) => <R1, W1, E1>(
  ma: Kind<M, S, R1, W1, E1, O.Option<A>>
) => Kind<M, S, R1 & R2 & R3, W2 | W3 | W1, E2 | E3 | E1, B | C>
/**
 * @since 3.0.0
 */
export declare const getOrElse: <F extends HKT>(
  F: Functor<F>
) => <B>(onNone: Lazy<B>) => <S, R, W, E, A>(fa: Kind<F, S, R, W, E, O.Option<A>>) => Kind<F, S, R, W, E, B | A>
/**
 * @since 3.0.0
 */
export declare const getOrElseE: <M extends HKT>(
  M: Monad<M>
) => <S, R2, W2, E2, B>(
  onNone: Lazy<Kind<M, S, R2, W2, E2, B>>
) => <R1, W1, E1, A>(ma: Kind<M, S, R1, W1, E1, O.Option<A>>) => Kind<M, S, R1 & R2, W2 | W1, E2 | E1, B | A>
/**
 * @since 3.0.0
 */
export declare const chainNullableK: <M extends HKT>(
  M: Monad<M>
) => <A, B>(
  f: (a: A) => B | null | undefined
) => <S, R, W, E>(ma: Kind<M, S, R, W, E, O.Option<A>>) => Kind<M, S, R, W, E, O.Option<NonNullable<B>>>
/**
 * @since 3.0.0
 */
export declare const chainOptionK: <M extends HKT>(
  M: Monad<M>
) => <A, B>(
  f: (a: A) => O.Option<B>
) => <S, R, W, E>(ma: Kind<M, S, R, W, E, O.Option<A>>) => Kind<M, S, R, W, E, O.Option<B>>
/**
 * @since 3.0.0
 */
export declare function map<F extends HKT>(
  F: Functor<F>
): <A, B>(f: (a: A) => B) => <S, R, W, E>(fa: Kind<F, S, R, W, E, Option<A>>) => Kind<F, S, R, W, E, Option<B>>
/**
 * @since 3.0.0
 */
export declare const ap: <F extends HKT>(
  F: Apply<F>
) => <S, R2, W2, E2, A>(
  fa: Kind<F, S, R2, W2, E2, O.Option<A>>
) => <R1, W1, E1, B>(
  fab: Kind<F, S, R1, W1, E1, O.Option<(a: A) => B>>
) => Kind<F, S, R1 & R2, W2 | W1, E2 | E1, O.Option<B>>
/**
 * @since 3.0.0
 */
export declare const chain: <M extends HKT>(
  M: Monad<M>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<M, S, R, W, E, O.Option<B>>
) => (ma: Kind<M, S, R, W, E, O.Option<A>>) => Kind<M, S, R, W, E, O.Option<B>>
/**
 * @since 3.0.0
 */
export declare const alt: <M extends HKT>(
  M: Monad<M>
) => <S, R2, W2, E2, B>(
  second: Lazy<Kind<M, S, R2, W2, E2, O.Option<B>>>
) => <R1, W1, E1, A>(
  first: Kind<M, S, R1, W1, E1, O.Option<A>>
) => Kind<M, S, R1 & R2, W2 | W1, E2 | E1, O.Option<B | A>>
