/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import type { Bifunctor as Bifunctor_ } from './Bifunctor'
import type { Chain } from './Chain'
import type { FromIO } from './FromIO'
import type { FromReader } from './FromReader'
import type { FromTask } from './FromTask'
import { FromWriter as FromWriter_ } from './FromWriter'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import type { IO } from './IO'
import type { Monad } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import type { Reader } from './Reader'
import * as RT from './ReaderTask'
import type { Semigroup } from './Semigroup'
import type { Task } from './Task'
import type { Writer } from './Writer'
import ReaderTask = RT.ReaderTask
/**
 * @category model
 * @since 3.0.0
 */
export interface ReaderTaskWriter<R, W, A> extends Reader<R, Task<Writer<W, A>>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromReader: <W>(w: W) => <R, A>(fa: Reader<R, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromReaderTask: <W>(w: W) => <R, A>(a: ReaderTask<R, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTaskWriter: <W, A, R = unknown>(a: Task<Writer<W, A>>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <W>(w: W) => <A, R = unknown>(fa: IO<A>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <W>(w: W) => <A, R = unknown>(fa: Task<A>) => ReaderTaskWriter<R, W, A>
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <W, R>(w: W) => ReaderTaskWriter<R, W, void>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReaderTaskWriter: <R1, R2, W, A>(
  f: (r1: R1) => ReaderTaskWriter<R2, W, A>
) => ReaderTaskWriter<R1 & R2, W, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromWriter: <W, A, R = unknown>(w: Writer<W, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromReaderWriter: <R, W, A>(fa: Reader<R, Writer<W, A>>) => ReaderTaskWriter<R, W, A>
/**
 * @since 3.0.0
 */
export declare const fst: <R, W, A>(t: ReaderTaskWriter<R, W, A>) => ReaderTask<R, A>
/**
 * @since 3.0.0
 */
export declare const snd: <R, W, A>(t: ReaderTaskWriter<R, W, A>) => ReaderTask<R, W>
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export declare const evaluate: <R, W, A>(t: ReaderTaskWriter<R, W, A>) => RT.ReaderTask<R, A>
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export declare const execute: <R, W, A>(t: ReaderTaskWriter<R, W, A>) => RT.ReaderTask<R, W>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(
  f: (r2: R2) => R1
) => <W, A>(ma: ReaderTaskWriter<R1, W, A>) => ReaderTaskWriter<R2, W, A>
/**
 * @since 3.0.0
 */
export declare const swap: <R, W, A>(t: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, A, W>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskWriterK: <A extends readonly unknown[], W, B>(
  f: (...a: A) => Task<Writer<W, B>>
) => <R = unknown>(...a: A) => ReaderTaskWriter<R, W, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromReaderWriterK: <A extends readonly unknown[], R, W, B>(
  f: (...a: A) => Reader<R, Writer<W, B>>
) => (...a: A) => ReaderTaskWriter<R, W, B>
/**
 * @since 3.0.0
 */
export declare const listen: <R, W, A>(fwa: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, readonly [A, W]>
/**
 * @since 3.0.0
 */
export declare const pass: <R, W, A>(
  fwa: ReaderTaskWriter<R, W, readonly [A, (w: W) => W]>
) => ReaderTaskWriter<R, W, A>
/**
 * @since 3.0.0
 */
export declare const listens: <W, B>(
  f: (w: W) => B
) => <R, A>(fwa: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, readonly [A, B]>
/**
 * @since 3.0.0
 */
export declare const censor: <W>(f: (w: W) => W) => <R, A>(fwa: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R, E>(fa: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, E, B>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapLeft: <E, G>(
  f: (e: E) => G
) => <R, A>(fea: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, G, A>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const bimap: <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <R>(fea: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, G, B>
/**
 * Maps a function over the first component of a `Writer`.
 *
 * Alias of [`map`](#map)
 *
 * @since 3.0.0
 */
export declare const mapFst: <A, B>(
  f: (a: A) => B
) => <R, E>(fa: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, E, B>
/**
 * Maps a function over the second component of a `Writer`.
 *
 * Alias of [`mapLeft`](#mapleft)
 *
 * @since 3.0.0
 */
export declare const mapSnd: <E, G>(
  f: (e: E) => G
) => <R, A>(fea: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, G, A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReaderTaskWriterF extends HKT {
  readonly type: ReaderTaskWriter<this['Contravariant1'], this['Covariant2'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReaderTaskWriterFFixedW<W> extends HKT {
  readonly type: ReaderTaskWriter<this['Contravariant1'], W, this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor_<ReaderTaskWriterF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<ReaderTaskWriterF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <R, E, B>(fab: ReaderTaskWriter<R, E, (a: A) => B>) => ReaderTaskWriter<R, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <W>(M: Monoid<W>) => Pointed<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <W>(A: Apply<RT.ReaderTaskF>, S: Semigroup<W>) => Apply<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <W>(
  A: Apply<RT.ReaderTaskF>,
  M: Monoid<W>
) => Applicative<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <W>(S: Semigroup<W>) => Chain<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <W>(M: Monoid<W>) => Monad<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromWriter: FromWriter_<ReaderTaskWriterF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromWriterK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Writer<E, B>
) => <R = unknown>(...a: A) => ReaderTaskWriter<R, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromReader: <W>(M: Monoid<W>) => FromReader<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromIO: <W>(M: Monoid<W>) => FromIO<ReaderTaskWriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromTask: <W>(M: Monoid<W>) => FromTask<ReaderTaskWriterFFixedW<W>>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, E, A>(
  fa: ReaderTaskWriter<R, E, A>
) => ReaderTaskWriter<
  R,
  E,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R, E>(
  fa: ReaderTaskWriter<R, E, A>
) => ReaderTaskWriter<
  R,
  E,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const tupled: <R, E, A>(fa: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, E, readonly [A]>
