/**
 * @since 3.0.0
 */
import { Bifunctor as Bifunctor_ } from './Bifunctor'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
/**
 * @since 3.0.0
 */
export interface Separated<E, A> {
  readonly left: E
  readonly right: A
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const separated: <E, A>(left: E, right: A) => Separated<E, A>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: <E, G, A, B>(f: (e: E) => G, g: (a: A) => B) => (fea: Separated<E, A>) => Separated<G, B>
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: <E, G>(f: (e: E) => G) => <A>(fea: Separated<E, A>) => Separated<G, A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface SeparatedF extends HKT {
  readonly type: Separated<this['Covariant2'], this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor_<SeparatedF>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <E>(fa: Separated<E, A>) => Separated<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<SeparatedF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Separated<E, (a: A) => B>) => Separated<E, B>
/**
 * @since 3.0.0
 */
export declare const left: <E, A>(s: Separated<E, A>) => E
/**
 * @since 3.0.0
 */
export declare const right: <E, A>(s: Separated<E, A>) => A
