/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import type { Bifunctor as Bifunctor_ } from './Bifunctor'
import type { Chain } from './Chain'
import type { Either } from './Either'
import { FromEither as FromEither_ } from './FromEither'
import { FromIO as FromIO_ } from './FromIO'
import { FromTask as FromTask_ } from './FromTask'
import { FromThese as FromThese_ } from './FromThese'
import { Lazy } from './function'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import type { IO } from './IO'
import type { IOEither } from './IOEither'
import type { Monad } from './Monad'
import type { Option } from './Option'
import type { Pointed as Pointed_ } from './Pointed'
import type { Predicate } from './Predicate'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import * as T from './Task'
import * as TH from './These'
import These = TH.These
import Task = T.Task
/**
 * @category model
 * @since 3.0.0
 */
export interface TaskThese<E, A> extends Task<These<E, A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const left: <E, A = never>(e: E) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const right: <A, E = never>(a: A) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const both: <E, A>(e: E, a: A) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightTask: <A, E = never>(ma: Task<A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftTask: <E, A = never>(me: Task<E>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightIO: <A, E = never>(ma: IO<A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftIO: <E, A = never>(me: IO<E>) => TaskThese<E, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <E, A>(fa: IOEither<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromEither: <E, A>(fa: Either<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromThese: <E, A>(fa: TH.These<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <A, E = never>(fa: IO<A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <A, E = never>(fa: T.Task<A>) => TaskThese<E, A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <E, B, A, C = B, D = B>(
  onLeft: (e: E) => B,
  onRight: (a: A) => C,
  onBoth: (e: E, a: A) => D
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B | C | D>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchE: <E, B, A, C = B, D = B>(
  onLeft: (e: E) => T.Task<B>,
  onRight: (a: A) => T.Task<C>,
  onBoth: (e: E, a: A) => T.Task<D>
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B | C | D>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <E, A>(ma: T.Task<TH.These<E, A>>) => T.Task<TH.These<A, E>>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <E>(fa: TaskThese<E, A>) => TaskThese<E, B>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: <E, G, A, B>(f: (e: E) => G, g: (a: A) => B) => (fea: TaskThese<E, A>) => TaskThese<G, B>
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: <E, G>(f: (e: E) => G) => <A>(fea: TaskThese<E, A>) => TaskThese<G, A>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, E = never>(a: A) => TaskThese<E, A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface TaskTheseF extends HKT {
  readonly type: TaskThese<this['Covariant2'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface TaskTheseFFixedE<E> extends HKT {
  readonly type: TaskThese<E, this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <E>(A: Apply<T.TaskF>, S: Semigroup<E>) => Apply<TaskTheseFFixedE<E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <E>(A: Apply<T.TaskF>, S: Semigroup<E>) => Applicative<TaskTheseFFixedE<E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <E>(S: Semigroup<E>) => Chain<TaskTheseFFixedE<E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <E>(S: Semigroup<E>) => Monad<TaskTheseFFixedE<E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<TaskTheseF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: TaskThese<E, (a: A) => B>) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<TaskTheseF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor_<TaskTheseF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: FromEither_<TaskTheseF>
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <E>(onNone: Lazy<E>) => <A>(fa: Option<A>) => TaskThese<E, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <E>(
  onNone: Lazy<E>
) => <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => Option<B>) => (...a: A) => TaskThese<E, B>
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <A, B extends A>(refinement: Refinement<A, B>): (a: A) => TaskThese<A, B>
  <A>(predicate: Predicate<A>): <B extends A>(b: B) => TaskThese<B, B>
  <A>(predicate: Predicate<A>): (a: A) => TaskThese<A, A>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromThese: FromThese_<TaskTheseF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTheseK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => TH.These<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIO_<TaskTheseF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => IO<B>
) => <E>(...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromTask: FromTask_<TaskTheseF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => T.Task<B>
) => <E = never>(...a: A) => TaskThese<E, B>
/**
 * @since 3.0.0
 */
export declare const toTuple2: <E, A>(e: Lazy<E>, a: Lazy<A>) => (fa: TaskThese<E, A>) => T.Task<readonly [E, A]>
/**
 * @since 3.0.0
 */
export declare const ApT: TaskThese<never, readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: ReadonlyNonEmptyArray<A>) => TaskThese<E, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: ReadonlyNonEmptyArray<A>) => TaskThese<E, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
