/**
 * @since 3.0.0
 */
import type { Comonad } from './Comonad'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import type { Monoid } from './Monoid'
/**
 * @category model
 * @since 3.0.0
 */
export interface Traced<W, A> {
  (w: W): A
}
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <W>(fa: Traced<W, A>) => Traced<W, B>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface TracedF extends HKT {
  readonly type: Traced<this['Invariant1'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface TracedFFixedW<W> extends HKT {
  readonly type: Traced<W, this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<TracedF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <W, B>(fab: Traced<W, (a: A) => B>) => Traced<W, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getComonad: <W>(monoid: Monoid<W>) => Comonad<TracedFFixedW<W>>
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
export declare const tracks: <W>(M: Monoid<W>) => <A>(f: (a: A) => W) => (fa: Traced<W, A>) => A
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
export declare const listen: <W, A>(fa: Traced<W, A>) => Traced<W, readonly [A, W]>
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const listens: <W, B>(f: (w: W) => B) => <A>(pa: Traced<W, A>) => Traced<W, readonly [A, B]>
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
export declare const censor: <W>(f: (p: W) => W) => <A>(pa: Traced<W, A>) => Traced<W, A>
