/**
 * @since 3.0.0
 */
import { Apply } from './Apply'
import type { Chain } from './Chain'
import type { FromIO } from './FromIO'
import type { FromTask } from './FromTask'
import { Functor } from './Functor'
import type { HKT, Kind } from './HKT'
import type { IO } from './IO'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import type { Semigroup } from './Semigroup'
import type { Task } from './Task'
import * as W from './Writer'
import Writer = W.Writer
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function fromF<F extends HKT>(
  F: Functor<F>
): <W>(w: W) => <S, R, FW, E, A>(fa: Kind<F, S, R, FW, E, A>) => Kind<F, S, R, FW, E, Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <F extends HKT>(
  F: Functor<F>,
  FT: FromIO<F>
) => <W>(w: W) => <A, S, R = unknown, FW = never, E = never>(fa: IO<A>) => Kind<F, S, R, FW, E, W.Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <F extends HKT>(
  F: Functor<F>,
  FT: FromTask<F>
) => <W>(w: W) => <A, S, R = unknown, FW = never, E = never>(fa: Task<A>) => Kind<F, S, R, FW, E, W.Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <F extends HKT>(
  F: Pointed<F>
) => <W, S, R = unknown, FW = never, E = never>(w: W) => Kind<F, S, R, FW, E, W.Writer<W, void>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function map<F extends HKT>(
  F: Functor<F>
): <A, B>(
  f: (a: A) => B
) => <S, R, FW, E, W>(fa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function of<F extends HKT, W>(
  F: Pointed<F>,
  M: Monoid<W>
): <A, S, R = unknown, FW = never, E = never>(a: A) => Kind<F, S, R, FW, E, Writer<W, A>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const ap: <F extends HKT, W>(
  F: Apply<F>,
  S: Semigroup<W>
) => <S, R2, FW2, E2, A>(
  fa: Kind<F, S, R2, FW2, E2, W.Writer<W, A>>
) => <R1, FW1, E1, B>(
  fab: Kind<F, S, R1, FW1, E1, W.Writer<W, (a: A) => B>>
) => Kind<F, S, R1 & R2, FW2 | FW1, E2 | E1, W.Writer<W, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const chain: <M extends HKT, W>(
  M: Chain<M>,
  S: Semigroup<W>
) => <A, S, R1, FW1, E1, B>(
  f: (a: A) => Kind<M, S, R1, FW1, E1, W.Writer<W, B>>
) => <R2, FW2, E2>(
  ma: Kind<M, S, R2, FW2, E2, W.Writer<W, A>>
) => Kind<M, S, R1 & R2, FW1 | FW2, E1 | E2, W.Writer<W, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function bimap<F extends HKT>(
  F: Functor<F>
): <W, G, A, B>(
  mapSnd: (w: W) => G,
  mapFst: (a: A) => B
) => <S, R, FW, E>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<G, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function mapLeft<F extends HKT>(
  F: Functor<F>
): <W, G>(
  mapSnd: (w: W) => G
) => <S, R, FW, E, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<G, A>>
/**
 * @since 3.0.0
 */
export declare function fst<F extends HKT>(
  F: Functor<F>
): <S, R, FW, E, W, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, A>
/**
 * @since 3.0.0
 */
export declare function snd<F extends HKT>(
  F: Functor<F>
): <S, R, FW, E, W, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, W>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare function swap<F extends HKT>(
  F: Functor<F>
): <S, R, FW, E, W, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<A, W>>
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function listen<F extends HKT>(
  F: Functor<F>
): <S, R, FW, E, W, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, readonly [A, W]>>
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function pass<F extends HKT>(
  F: Functor<F>
): <S, R, FW, E, W, A>(
  fwa: Kind<F, S, R, FW, E, Writer<W, readonly [A, (w: W) => W]>>
) => Kind<F, S, R, FW, E, Writer<W, A>>
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function listens<F extends HKT>(
  F: Functor<F>
): <W, B>(
  f: (w: W) => B
) => <S, R, FW, E, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, readonly [A, B]>>
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function censor<F extends HKT>(
  F: Functor<F>
): <W>(
  f: (w: W) => W
) => <S, R, FW, E, A>(fwa: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, A>>
